<?php

namespace Drupal\scenarios_contenthub;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\acquia_contenthub\ContentHubCommonActions;
use Drupal\Core\StreamWrapper\PublicStream;
use Acquia\ContentHubClient\CDFDocument;
use Acquia\ContentHubClient\CDF\CDFObject;
use Drupal\Component\Serialization\Json;
use Drupal\scenarios\ScenariosHandler;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class ScenariosContentHub.
 *
 * @package Drupal\scenarios_contenthub
 */
class ScenariosContentHub implements ContainerInjectionInterface {

  /**
   * Drupal\scenarios\ScenariosHandler definition.
   *
   * @var \Drupal\scenarios\ScenariosHandler
   */
  protected $scenariosHandler;

  /**
   * Drupal\acquia_contenthub\ContentHubCommonActions definition.
   *
   * @var \Drupal\acquia_contenthub\ContentHubCommonActions
   */
  protected $commonActions;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  public $entityTypeManager;

  /**
   * Drupal\Core\Entity\EntityTypeBundleInfoInterface
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityBundleInfo;

  /**
   * Constructor.
   *
   * @var \Drupal\scenarios\ScenariosHandler
   * @var \Drupal\acquia_contenthub\ContentHubCommonActions
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  public function __construct(ScenariosHandler $scenarios_handler, ContentHubCommonActions $common_actions, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_bundle_info) {
    $this->scenariosHandler = $scenarios_handler;
    $this->commonActions = $common_actions;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityBundleInfo = $entity_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('scenarios_handler'),
      $container->get('acquia_contenthub_common_actions'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * @param $scenario
   * @param $cdf
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \ReflectionException
   */
  public function import($scenario, $cdf) {
    // Retrieve the scenario information.
    if ($info = scenarios_info($scenario)) {
      $scenario_dir = drupal_get_path($info['type'], $scenario);
      // Run the Content Hub CDF import.
      foreach ($cdf as $cdf_dir) {
        $cdf_path = $scenario_dir . $cdf_dir;
        $files = \Drupal::service('file_system')->scanDirectory($cdf_path, '/.*\.json$/');
        foreach ($files as $file) {
          $this->importCDF($file);
        }
      }
    }
    else {
      $this->scenariosHandler->setError(t('The scenario @scenario does not exist for cdf import.', ['@scenario' => $scenario]));
    }
  }

  /**
   * @param $file
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \ReflectionException
   */
  public function importCDF($file) {
    $host = str_replace('/', '\/',\Drupal::request()->getSchemeAndHttpHost());
    $json = str_replace('%scenarios-replace-host%', $host, file_get_contents($file->uri));
    $filepath = str_replace('/', '\/', PublicStream::basePath());
    $json = str_replace('%scenarios-replace-filepath%', $filepath, $json);
    $data = Json::decode($json);
    $document_parts = [];
    foreach ($data['entities'] as $entity) {
      $document_parts[] = CDFObject::fromArray($entity);
    }
    $cdf_document = new CDFDocument(...$document_parts);
    if ($this->commonActions->importEntityCdfDocument($cdf_document)) {
      $this->scenariosHandler->setMessage(t('Imported entities via @filename', ['@filename' => $file->filename]));
    }
  }

  /**
   * @param $filename
   * @param $types
   *
   * @return \Drupal\file\FileInterface|false
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function export($filename, $types = []) {
    $entities = [];
    // Loop the entity types extracting items via bundle type.
    foreach ($types as $type => $bundles) {
      $properties = [];
      if ($bundle_key = $this->entityTypeManager->getDefinition($type)->getKey('bundle')) {
        $properties[$bundle_key] = $bundles;
      }
      $results = $this->entityTypeManager->getStorage($type)->loadByProperties($properties);
      // Add results to the entity list (unless they're user/0 or user/1).
      foreach ($results as $key => $result) {
        if ($type == 'user' && $result->id() <= 1) {
          unset($results[$key]);
        }
      }
      foreach ($results as $result) {
        $entities[] = $result;
      }
    }
    // Create the local CDF containing entities and their calculated dependencies.
    $document = $this->commonActions->getLocalCdfDocument(...$entities);
    $host = str_replace('/', '\/',\Drupal::request()->getSchemeAndHttpHost());
    $json = str_replace($host, '%scenarios-replace-host%', $document->toString());
    $filepath = str_replace('/', '\/', PublicStream::basePath());
    $json = str_replace($filepath, '%scenarios-replace-filepath%', $json);
    // Save the json response to a file.
    return file_save_data($json, 'public://' . $filename);
  }

}
