<?php

namespace Drupal\scenarios_migrate;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Plugin\MigrationPluginManager;
use Drupal\migrate\MigrateMessage;
use Drupal\migrate_tools\DrushLogMigrateMessage;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\scenarios\ScenariosHandler;

/**
 * Class ScenariosMigrateHandler.
 *
 * @package Drupal\scenarios
 */
class ScenariosMigrateHandler implements ContainerInjectionInterface {

  /**
   * Drupal\migrate\Plugin\MigrationPluginManager definition.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManager
   */
  protected $migrationPluginManager;

  /**
   * Drupal\scenarios\ScenariosHandler definition.
   *
   * @var \Drupal\scenarios\ScenariosHandler
   */
  protected $scenariosHandler;

  /**
   * Constructor.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManager
   * @var \Drupal\scenarios\ScenariosHandler
   */
  public function __construct(MigrationPluginManager $migration_plugin_manager, ScenariosHandler $scenarios_handler) {
    $this->migrationPluginManager = $migration_plugin_manager;
    $this->scenariosHandler = $scenarios_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.migration'),
      $container->get('scenarios_handler')
    );
  }


  /**
   * Retrieves the logger.
   *
   * @param string $alias
   *   The Drush alias context.
   *
   * @return \Drupal\migrate\MigrateMessage|\Drupal\migrate_tools\DrushLogMigrateMessage
   */
  public function getLog($alias) {
    if ($alias != NULL && function_exists('drush_log')) {
      return new DrushLogMigrateMessage();
    }
    else {
      return new MigrateMessage();
    }
  }

  /**
   * Processes the batch.
   *
   * @param string $alias
   *   The Drush alias context.
   */
  public function processBatch($alias) {
    if (batch_get()) {
      if ($alias !== NULL && function_exists('drush_backend_batch_process')) {
        drush_backend_batch_process();
      }
      else {
        batch_process();
      }
    }
  }

  /**
   * @param $command
   * @param $migrations
   * @param $alias
   *
   * @return bool
   * @throws \Drupal\migrate\MigrateException
   */
  public function processMigrations($command, $migrations, $alias) {
    $migration_manager = \Drupal::service('plugin.manager.migration');
    $migration_manager->clearCachedDefinitions();
    // Return the correct log for the Migrate Executable.
    $log = $this->getLog($alias);
    // Set default value for return.
    $result = FALSE;
    // If we have pending batches, process them now.
    $this->processBatch($alias);
    // Run the migrations in the provided order.
    foreach ($migrations as $migration) {
      $migration = $migration_manager->createInstance($migration);
      $executable = new MigrateExecutable($migration, $log);
      switch ($command) {
        case "import":
          if ($execute = $executable->import()) {
            // Invoke the 'scenarios_migration_finished' hook from the global
            // container in case a scenario dependency implements that hook.
            \Drupal::moduleHandler()->invokeAll('scenarios_migration_finished', [$migration]);
          }
          break;
        case "rollback":
          $execute = $executable->rollback();
          break;
        default:
          $execute = FALSE;
      }
      // Return the migration results.
      if ($execute) {
        $this->scenariosHandler->setMessage(t('Executed @command for "@label" migration.', ['@label' => $migration->label(), '@command' => $command]));
        $result = TRUE;
      }
      else {
        $this->scenariosHandler->setError(t('Migration "@label" failed to execute @command', ['@label' => $migration->label(), '@command' => $command]));
        $result = FALSE;
      }
    }
    return $result;
  }

}
