<?php

namespace Drupal\scenarios_ui\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\InfoParserException;
use Drupal\Core\Extension\InfoParser;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Render\Renderer;
use Drupal\scenarios\ScenariosHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ScenariosUIForm.
 */
class ScenariosUIForm extends FormBase {

  /**
   * Drupal\scenarios\ScenariosHandler definition.
   *
   * @var \Drupal\scenarios\ScenariosHandler
   */
  protected $scenariosHandler;

  /**
   * Drupal\Core\Extension\InfoParser definition.
   *
   * @var \Drupal\Core\Extension\InfoParser
   */
  protected $infoParser;

  /**
   * Drupal\Core\Render\Renderer definition.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Drupal\Core\Extension\ModuleHandler definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Drupal\Core\Extension\ModuleInstallerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleInstallerInterface
   */
  protected $moduleInstaller;

  /**
   * ScenariosUIForm constructor.
   *
   * @param \Drupal\scenarios\ScenariosHandler $scenarios_handler
   * @param \Drupal\Core\Extension\InfoParser $info_parser
   * @param \Drupal\Core\Render\Renderer $drupal_renderer
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   * @param \Drupal\Core\Extension\ModuleInstallerInterface $module_installer
   */
  public function __construct(ScenariosHandler $scenarios_handler, InfoParser $info_parser, Renderer $drupal_renderer, ModuleHandler $module_handler, ModuleInstallerInterface $module_installer) {
    $this->scenariosHandler = $scenarios_handler;
    $this->infoParser = $info_parser;
    $this->renderer = $drupal_renderer;
    $this->moduleHandler = $module_handler;
    $this->moduleInstaller = $module_installer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('scenarios_handler'),
      $container->get('info_parser'),
      $container->get('renderer'),
      $container->get('module_handler'),
      $container->get('module_installer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'scenarios_ui_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    try {
      $modules = system_rebuild_module_data();
      uasort($modules, 'system_sort_modules_by_info_name');
    }
    catch (InfoParserException $e) {
      $this->messenger()->addError($this->t('Modules could not be listed due to an error: %error', ['%error' => $e->getMessage()]));
      $modules = [];
    }

    $options = [];
    $defaults = [];
    foreach ($modules as $key => $extension) {
      $info = $this->infoParser->parse($extension->getPathname());
      if (isset($info['scenario'])) {
        $scenario_image = [
          '#theme' => 'image',
          '#theme_wrappers' => [
            'container' => [
              '#attributes' => [
                'class' => 'scenario--image'
              ],
            ],
          ],
          '#uri' => $extension->getPath() . '/scenario.png',
          '#attributes' => [
            'alt' => $info['description']
          ],
          '#attached' => [
            'library' => [
              'scenarios_ui/manage'
            ]
          ]
        ];
        $image = $this->renderer->render($scenario_image);
        $text = $this->t('<h5>@name</h5>', ['@name' => $info['name']]);
        $options[$key] = $image . $text;
        if ($this->moduleHandler->moduleExists($extension->getName())) {
          $defaults[$key] = $key;
        }
      }
    }

    $form['scenarios'] = [
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $defaults
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit')
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @throws \Drupal\Core\Extension\MissingDependencyException
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $scenarios = $form_state->getValue('scenarios');
    foreach ($scenarios as $key => $value) {
      $exists = $this->moduleHandler->moduleExists($key);
      if ($value === $key && $exists != TRUE) {
        $this->moduleInstaller->install([$key]);
      }
      if ($value === 0 && $exists == TRUE) {
        if ($this->scenariosHandler->scenarioUninstall($key)) {
          $this->moduleInstaller->uninstall([$key]);
        }
      }
    }
  }

}
