<?php

/**
 * @file
 * Contains \Drupal\scenarios_contenthub\Form\ScenariosImportForm.
 */
namespace Drupal\scenarios\Form;

use Drupal\Core\Link;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandler;

/**
 * Implements the Scenarios Settings form.
 */
class ScenariosSettingsForm extends FormBase {

  /**
   * Drupal\Core\Extension\ModuleHandler definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Entity\EntityTypeBundleInfoInterface
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityBundleInfo;

  /**
   * ExampleForm constructor.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  public function __construct(ModuleHandler $module_handler, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_bundle_info) {
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityBundleInfo = $entity_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'scenarios_settings_form';
  }

  /**
   * {@inheritdoc}.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $safe_list = $this->config('scenarios.settings')->get('safe');
    $options = [];
    $types = $this->entityTypeManager->getDefinitions();
    foreach ($types as $type => $type_object) {
      if ($type_object->getGroup() == 'content' && in_array($type, $safe_list)) {
        $bundles = $this->entityBundleInfo->getBundleInfo($type);
        foreach ($bundles as $bundle => $info) {
          $string = $type . ':' . $bundle;
          $options[$string] = $string . " (" . $type_object->getLabel() . ' - ' . $info['label'] . ")";
        }
      }
    }
    $default = $this->config('scenarios.settings')->get('types');
    $default_value = [];
    foreach ($default as $type => $bundles) {
      foreach ($bundles as $bundle) {
        $key = $type . ':' . $bundle;
        $default_value[$key] = $key;
      }
    }
    $form['types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select bundle types to export'),
      '#options' => $options,
      '#default_value' => $default_value,
    ];
    $form['info'] = [
      '#title' => 'WARNING - Scenario exports are experimental code!',
      '#type' => 'details',
      '#open' => TRUE
    ];
    $form['info']['select_notes'] = [
      '#type' => 'item',
      '#plain_text' => $this->t('All entities of the selected types are included in json exports.')
    ];
    $form['info']['php_notes'] = [
      '#type' => 'item',
      '#plain_text' => $this->t('Selecting a large amount of entities may exhaust PHP Memory on export.')
    ];
    $form['info']['extra_notes'] = [
      '#type' => 'item',
      '#plain_text' => $this->t('Unselected entities may be added during dependency calculation. (ex: node:article might add taxonomy_term:tags)')
    ];
    $form['info']['export_download'] = [
      '#type' => 'item',
      '#markup' => $this->t('The "download" button saves the configuration before exporting a scenario.json file.')
    ];
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced Settings'),
      '#open' => FALSE
    ];
    $type_options = [];
    foreach ($types as $type => $type_object) {
      if ($type_object->getGroup() == 'content') {
        $type_options[$type] = $type;
      }
    }
    $form['advanced']['safe'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Entity types available in export configuration'),
      '#options' => $type_options,
      '#default_value' => $safe_list,
    ];
    $form['advanced']['safe_info'] = [
      '#type' => 'item',
      '#markup' => $this->t('Selecting an entity type will modify the available bundle checkboxes. Save Configuration to reload the form after changing these values.')
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Configuration'),
      '#button_type' => 'primary',
      '#name' => 'save'
    ];
    $form['actions']['download'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download Export'),
      '#name' => 'download'
    ];

    // Disable functionality when the CH module is not installed.
    if (!$this->moduleHandler->moduleExists('scenarios_contenthub')) {
      $form['types']['#disabled'] = TRUE;
      $form['types']['#description'] = $this->t('The scenarios_contenthub module must be enabled to export data.');
      $form['actions']['#access'] = FALSE;
      $form['advanced']['#access'] = FALSE;
      $form['info']['#access'] = FALSE;
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $settings = $this->configFactory()->getEditable('scenarios.settings');
    $safe_list = $form_state->getValue('safe');
    foreach ($safe_list as $key => $value) {
      if ($value == '0') {
        unset($safe_list[$key]);
      }
    }
    $settings->set('safe', $safe_list)->save();
    $types = [];
    // Make a list of possible entity types available.
    foreach ($safe_list as $type) {
      $types[$type] = [];
    }
    // Grab the selected values from the form and aggregate configuration value.
    $selected = $form_state->getValue('types');
    foreach ($selected as $key => $value) {
      if ($value != '0') {
        $var = explode(':', $value);
        $types[$var[0]][] = $var[1];
      }
    }
    // Remove any entity types left unselected from 'types' config value.
    foreach ($types as $type => $value) {
      if ($value === []) {
        unset($types[$type]);
      }
    }
    $settings->set('types', $types)->save();
    $trigger = $form_state->getTriggeringElement();
    if ($trigger['#name'] == 'download') {
      $form_state->setRedirect('scenarios_contenthub.scenario_builder_controller_download');
    }
  }
}
