<?php

namespace Drupal\scenarios;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ScenariosHandler.
 *
 * @package Drupal\scenarios
 */
class ScenariosHandler implements ContainerInjectionInterface {

  /**
   * Drupal\Core\Extension\ModuleHandler definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Drupal\Core\Config\ConfigFactory definition
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Constructor.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   * @var \Drupal\Core\ProxyClass\Extension\ModuleInstaller
   * @var \Drupal\Core\Config\ConfigFactory
   */
  public function __construct(ModuleHandler $module_handler, ConfigFactory $config_factory) {
    $this->moduleHandler = $module_handler;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('config.factory')
    );
  }

  /**
   * Sets a message to display to the user.
   *
   * @param string|\Drupal\Component\Render\MarkupInterface $message
   *   The translated message to be displayed to the user. For consistency with
   *   other messages, it should begin with a capital letter and end with a
   *   period.
   * @param string $type
   *   (optional) The message's type. Defaults to 'status'. These values are
   *   supported:
   *   - 'status'
   *   - 'warning'
   *   - 'error'
   * @param bool $repeat
   *   (optional) If this is FALSE and the message is already set, then the
   *   message won't be repeated. Defaults to FALSE.
   */
  public function setMessage($message, $type = 'status', $repeat = FALSE) {
    if (PHP_SAPI === 'cli' && function_exists('drush_log')) {
      $type = ($type == 'status' ? 'ok' : $type);
      drush_log($message, $type);
    }
    else {
      \Drupal::messenger()->addMessage($message, $type, $repeat);
    }
  }

  /**
   * Sets an error message to display to the user.
   *
   * @param string|\Drupal\Component\Render\MarkupInterface $message
   *   The translated error message to be displayed to the user. For consistency
   *   with other messages, it should begin with a capital letter and end with a
   *   period.
   */
  public function setError($message) {
    if (PHP_SAPI === 'cli' && function_exists('drush_set_error')) {
      drush_set_error('ERROR', $message);
    }
    else {
      \Drupal::messenger()->addMessage($message, 'error');
    }
  }

  /**
   * Rebuilds the site cache.
   *
   * @param string $alias
   *   The Drush alias context.
   */
  public function cacheRebuild($alias) {
    if ($alias !== NULL && function_exists('drush_invoke_process')) {
      drush_invoke_process($alias, 'cache-rebuild');
    }
    else {
      drupal_flush_all_caches();
    }
  }

  /**
   * Retrieves the Drush alias context.
   *
   * @return null|string
   *   The Drush alias context or NULL if the context does not exist.
   */
  public function getAlias() {
    $alias = NULL;
    if (PHP_SAPI === 'cli' && function_exists('drush_get_context')) {
      $alias_context = drush_get_context('alias');
      $alias = !empty($alias_context) ? $alias_context : '@self';
    }
    return $alias;
  }

  /**
   * @param $scenario
   */
  public function scenarioEnable($scenario) {
    // Retrieve the scenario information.
    if ($info = scenarios_info($scenario)) {
      //Check to see if the scenario can be enabled.
      if (!isset($info['label'])) {
        $this->setError(t("Unable to enable the Scenario due to missing label."));
        return;
      }
      // Inform the user that the scenario is being enabled.
      $this->setMessage(t('Enabling the @scenario scenario.', ['@scenario' => $info['label']]));
      // Get the Drush alias.
      $alias = $this->getAlias();
      // Get the current install profile directory for the scenario.
      $scenario_dir = drupal_get_path($info['type'], $scenario);
      // Copy the unmanaged files.
      if (isset($info['files'])) {
        $count = 0;
        foreach ($info['files'] as $dir => $destination) {
          $files_dir = $scenario_dir . $dir;
          $files = \Drupal::service('file_system')->scanDirectory($files_dir, '/.*\.(json|jpg|png)$/');
          if (\Drupal::service('file_system')->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY)) {
            foreach ($files as $file) {
              try {
                \Drupal::service('file_system')->copy($file->uri, $destination, FileSystemInterface::EXISTS_REPLACE);
                $count++;
              }
              catch (FileException $e) {
                $replace = [
                  '@uri' => $file->uri,
                  '@path' => $destination,
                  '@error' => $e
                ];
                $this->setError(t("Unable to copy @uri to @path with Error: @error", $replace));
              }
            }
          }
        }
        $this->setMessage(t("@count files copied successfully.", ['@count' => $count]));
      }

      // Run the Migrations where applicable.
      if (isset($info['migrations']) && $this->moduleHandler->moduleExists('scenarios_migrate')) {
        // Retrieve the scenario migrations.
        $migrations = $info['migrations'];
        // Process the migrations.
        \Drupal::service('scenarios_migrate_handler')->processMigrations('import', $migrations, $alias);
      }

      // Run the Default Content Deploy imports where available.
      if (isset($info['imports']) && $this->moduleHandler->moduleExists('default_content_deploy')) {
        $importer = \Drupal::service('default_content_deploy.importer');
        // Load the Default Content Deploy configuration.
        $config = $this->configFactory->getEditable('default_content_deploy.content_directory');
        // Run the imports over multiple content directories if necessary.
        foreach ($info['imports'] as $dir) {
          $import_dir = '../docroot/' . $scenario_dir . $dir;
          $config->set('content_directory', $import_dir)->save();
          if ($importer->deployContent($force_update = FALSE, $writeEnable = TRUE)) {
            $this->setMessage(t('Default content for @scenario imported.', ['@scenario' => $scenario]));
          }
          if ($importer->importUrlAliases()) {
            $this->setMessage(t('URL aliases for @scenario imported.', ['@scenario' => $scenario]));
          }
        }
      }

      // Run the Acquia Content Hub CDF import where available.
      if (isset($info['cdf']) && $this->moduleHandler->moduleExists('scenarios_contenthub')) {
        \Drupal::service('scenarios_contenthub')->import($scenario, $info['cdf']);
      }

      // Provide a hook that allows modules to act after the scenario has been
      // enabled.
      $this->moduleHandler->invokeAll('scenarios_post_enable', [$scenario]);
    }
    else {
      $this->setError(t('The scenario @scenario does not exist.', ['@scenario' => $scenario]));
    }
  }

  /**
   * @param $scenario
   */
  public function scenarioUninstall($scenario) {
    // Retrieve the scenario information.
    if ($info = scenarios_info($scenario)) {
      // Inform the user that the scenario is being uninstalled.
      $this->setMessage(t('Uninstalling the @scenario scenario.', ['@scenario' => $info['label']]));
      // Get the Drush alias.
      $alias = $this->getAlias();
      if (isset($info['migrations']) && $this->moduleHandler->moduleExists('scenarios_migrate')) {
        // Retrieve the scenario migrations and reverse their order.
        $migrations = array_reverse($info['migrations']);
        // Process the migrations.
        \Drupal::service('scenarios_migrate_handler')->processMigrations('rollback', $migrations, $alias);
      }
      // Provide a hook that allows modules to act after the scenario has been
      // uninstalled.
      $this->moduleHandler->invokeAll('scenarios_post_uninstall', [$scenario]);
    }
    else {
      $this->setError(t('The scenario @scenario does not exist.', ['@scenario' => $scenario]));
    }
  }

  /**
   * @param $scenario
   */
  public function scenarioReset($scenario) {
    // Retrieve the scenario information.
    if ($info = scenarios_info($scenario)) {
      // Inform the user that the scenario is being reset.
      $this->setMessage(t('Resetting the @scenario scenario.', ['@scenario' => $info['label']]));
      // Uninstall the scenario.
      $this->scenarioUninstall($scenario);
      // Enable the scenario.
      $this->scenarioEnable($scenario);
      // Provide a hook that allows modules to act after the scenario has been
      // reset.
      $this->moduleHandler->invokeAll('scenarios_post_reset', [$scenario]);
    }
    else {
      $this->setError(t('The scenario @scenario does not exist.', ['@scenario' => $scenario]));
    }
  }

}
